@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -33,6 +30,9 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include <gmime/gmime.h>
 
 
@@ -310,7 +310,7 @@ write_part (GMimeObject *part, const char *uid, const char *spec)
 		
 		ostream = g_mime_stream_file_new (fp);
 		g_mime_object_write_to_stream (GMIME_OBJECT (message), ostream);
-		g_mime_stream_unref (ostream);
+		g_object_unref (ostream);
 	} else if (GMIME_IS_PART (part)) {
 		buf = g_strdup_printf ("%s/%s.TEXT", uid, spec);
 		fp = fopen (buf, "wt");
@@ -320,8 +320,8 @@ write_part (GMimeObject *part, const char *uid, const char *spec)
 		istream = g_mime_data_wrapper_get_stream (GMIME_PART (part)->content);
 		
 		g_mime_stream_write_to_stream (istream, ostream);
-		g_mime_stream_unref (istream);
-		g_mime_stream_unref (ostream);
+		g_object_unref (istream);
+		g_object_unref (ostream);
 	}
 }
 
@@ -393,7 +393,7 @@ decode_qstring (unsigned char **in, unsigned char *inend)
 	if (inptr == inend)
 		return NULL;
 	
-	if (strncmp (inptr, "NIL", 3) != 0) {
+	if (strncmp ((const char *) inptr, "NIL", 3) != 0) {
 		g_assert (*inptr == '"');
 		inptr++;
 		start = inptr;
@@ -403,7 +403,7 @@ decode_qstring (unsigned char **in, unsigned char *inend)
 			inptr++;
 		}
 		
-		qstring = g_strndup (start, inptr - start);
+		qstring = g_strndup ((const char *) start, inptr - start);
 		unescape_qstring (qstring);
 		g_assert (*inptr == '"');
 		inptr++;
@@ -439,7 +439,6 @@ decode_params (unsigned char **in, unsigned char *inend)
 {
 	GMimeParam *params, *tail, *n;
 	unsigned char *inptr;
-	char *buf;
 	
 	inptr = *in;
 	params = NULL;
@@ -947,7 +946,7 @@ int main (int argc, char **argv)
 	
 	if (message) {
 		uid = g_strdup (message->message_id ? message->message_id : basename (argv[i]));
-		g_mkdir (uid, 0755);
+		g_mkdir (uid, 0777);
 		write_message (message, uid);
 		g_object_unref (message);
 		
