@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -26,6 +27,18 @@
 
 #include "gmime-filter-from.h"
 
+
+/**
+ * SECTION: gmime-filter-from
+ * @title: GMimeFilterFrom
+ * @short_description: Escape MBox From_ lines
+ * @see_also: #GMimeFilter
+ *
+ * A #GMimeFilter used for escaping MBox From_ lines using either the
+ * traditional ">From " or quoted-printable encoding.
+ **/
+
+
 static void g_mime_filter_from_class_init (GMimeFilterFromClass *klass);
 static void g_mime_filter_from_init (GMimeFilterFrom *filter, GMimeFilterFromClass *klass);
 static void g_mime_filter_from_finalize (GObject *object);
@@ -239,7 +252,7 @@ g_mime_filter_from_new (GMimeFilterFromMode mode)
 {
 	GMimeFilterFrom *new;
 	
-	new = g_object_new (GMIME_TYPE_FILTER_FROM, NULL, NULL);
+	new = g_object_new (GMIME_TYPE_FILTER_FROM, NULL);
 	new->midline = FALSE;
 	switch (mode) {
 	case GMIME_FILTER_FROM_MODE_ARMOR:
