@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -36,6 +36,15 @@
 #include "gmime-parse-utils.h"
 
 
+/**
+ * SECTION: gmime-message
+ * @title: GMimeMessage
+ * @short_description: Messages
+ * @see_also:
+ *
+ * A #GMimeMessage represents an rfc822 message.
+ **/
+
 static void g_mime_message_class_init (GMimeMessageClass *klass);
 static void g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass);
 static void g_mime_message_finalize (GObject *object);
@@ -49,8 +58,9 @@ static void message_remove_header (GMimeObject *object, const char *header);
 static char *message_get_headers (GMimeObject *object);
 static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream);
 
-static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string);
+static void message_add_recipients_from_string (GMimeMessage *message, const char *type, const char *string);
 
+static ssize_t write_structured (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_received (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_subject (GMimeStream *stream, const char *name, const char *value);
@@ -145,7 +155,7 @@ g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Subject", write_subject);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Received", write_received);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Message-Id", write_msgid);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "References", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "References", write_structured);
 }
 
 static gboolean
@@ -548,7 +558,7 @@ write_msgid (GMimeStream *stream, const char *name, const char *value)
 }
 
 static ssize_t
-write_addrspec (GMimeStream *stream, const char *name, const char *value)
+write_structured (GMimeStream *stream, const char *name, const char *value)
 {
 	char *unfolded, *folded;
 	ssize_t n;
@@ -563,6 +573,29 @@ write_addrspec (GMimeStream *stream, const char *name, const char *value)
 	return n;
 }
 
+static ssize_t
+write_addrspec (GMimeStream *stream, const char *name, const char *value)
+{
+	InternetAddressList *addrlist;
+	GString *str;
+	ssize_t n;
+	
+	str = g_string_new (name);
+	g_string_append (str, ": ");
+	
+	if (value && (addrlist = internet_address_parse_string (value))) {
+		internet_address_list_writer (addrlist, str);
+		internet_address_list_destroy (addrlist);
+	}
+	
+	g_string_append_c (str, '\n');
+	
+	n = g_mime_stream_write (stream, str->str, str->len);
+	g_string_free (str, TRUE);
+	
+	return n;
+}
+
 
 enum {
 	HEADER_FROM,
@@ -808,24 +841,29 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	GMimeMessage *message = (GMimeMessage *) object;
 	ssize_t nwritten, total = 0;
 	
-	if (!(message->mime_part && g_mime_header_has_raw (message->mime_part->headers))) {
-		/* write the content headers */
-		if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
-			return -1;
-		
-		total += nwritten;
-		
-		if (message->mime_part) {
+	if (message->mime_part) {
+		if (!g_mime_header_has_raw (message->mime_part->headers)) {
+			/* if the mime part has raw headers, then it contains the message
+			 * headers as well otherwise it doesn't, so write them now */
+			if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+				return -1;
+			
+			total += nwritten;
+			
 			if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
 				return -1;
 			
 			total += nwritten;
 		}
-	}
-	
-	if (message->mime_part) {
-		nwritten = g_mime_object_write_to_stream (message->mime_part, stream);
+		
+		if ((nwritten = g_mime_object_write_to_stream (message->mime_part, stream)) == -1)
+			return -1;
 	} else {
+		if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
+			return -1;
+		
+		total += nwritten;
+		
 		if ((nwritten = g_mime_stream_write (stream, "\n", 1)) == -1)
 			return -1;
 	}
@@ -976,7 +1014,7 @@ sync_recipient_header (GMimeMessage *message, const char *type)
  * #GMIME_RECIPIENT_TYPE_CC and #GMIME_RECIPIENT_TYPE_BCC.
  **/
 void
-g_mime_message_add_recipient (GMimeMessage *message, char *type, const char *name, const char *address)
+g_mime_message_add_recipient (GMimeMessage *message, const char *type, const char *name, const char *address)
 {
 	InternetAddressList *recipients;
 	InternetAddress *ia;
@@ -994,13 +1032,13 @@ g_mime_message_add_recipient (GMimeMessage *message, char *type, const char *nam
 	recipients = internet_address_list_append (recipients, ia);
 	internet_address_unref (ia);
 	
-	g_hash_table_insert (message->recipients, type, recipients);
+	g_hash_table_insert (message->recipients, (char *) type, recipients);
 	sync_recipient_header (message, type);
 }
 
 
 static void
-message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string)
+message_add_recipients_from_string (GMimeMessage *message, const char *type, const char *string)
 {
 	InternetAddressList *recipients, *addrlist;
 	
@@ -1012,7 +1050,7 @@ message_add_recipients_from_string (GMimeMessage *message, char *type, const cha
 		internet_address_list_destroy (addrlist);
 	}
 	
-	g_hash_table_insert (message->recipients, type, recipients);
+	g_hash_table_insert (message->recipients, (char *) type, recipients);
 }
 
 
@@ -1029,7 +1067,7 @@ message_add_recipients_from_string (GMimeMessage *message, char *type, const cha
  * specified in rfc822.
  **/
 void
-g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string)
+g_mime_message_add_recipients_from_string (GMimeMessage *message, const char *type, const char *string)
 {
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (type != NULL);
