@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -23,16 +24,18 @@
 #endif
 
 #include <string.h>
+#include <limits.h>
 #include <ctype.h>
 #include <errno.h>
 
 #include "gmime-param.h"
 #include "gmime-common.h"
 #include "gmime-table-private.h"
+#include "gmime-parse-utils.h"
+#include "gmime-iconv-utils.h"
 #include "gmime-charset.h"
 #include "gmime-utils.h"
 #include "gmime-iconv.h"
-#include "gmime-iconv-utils.h"
 
 
 #ifdef ENABLE_WARNINGS
@@ -44,6 +47,17 @@
 #define d(x)
 
 
+/**
+ * SECTION: gmime-param
+ * @title: GMimeParam
+ * @short_description: Content-Type and Content-Disposition parameters
+ * @see_also: #GMimeContentType
+ *
+ * A #GMimeParam is a parameter name/value pair as found on MIME
+ * header fields such as Content-Type and Content-Disposition.
+ **/
+
+
 static unsigned char tohex[16] = {
 	'0', '1', '2', '3', '4', '5', '6', '7',
 	'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
@@ -55,7 +69,7 @@ static unsigned char tohex[16] = {
  * @name: parameter name
  * @value: parameter value
  *
- * Creates a new GMimeParam node with name @name and value @value.
+ * Creates a new #GMimeParam node with name @name and value @value.
  *
  * Returns a new paramter structure.
  **/
@@ -73,50 +87,31 @@ g_mime_param_new (const char *name, const char *value)
 	return param;
 }
 
-static void
-decode_lwsp (const char **in)
-{
-	const char *inptr = *in;
-	
-	while (*inptr && (*inptr == '(' || is_lwsp (*inptr))) {
-		while (*inptr && is_lwsp (*inptr))
-			inptr++;
-		
-		/* skip over any comments */
-		if (*inptr == '(') {
-			int depth = 1;
-			
-			inptr++;
-			while (*inptr && depth) {
-				if (*inptr == '\\' && *(inptr + 1))
-					inptr++;
-				else if (*inptr == '(')
-					depth++;
-				else if (*inptr == ')')
-					depth--;
-				
-				inptr++;
-			}
-		}
-	}
-	
-	*in = inptr;
-}
+#define INT_OVERFLOW(x,d) (((x) > (INT_MAX / 10)) || ((x) == (INT_MAX / 10) && (d) > (INT_MAX % 10)))
 
 static int
 decode_int (const char **in)
 {
-	const char *inptr = *in;
-	int n = 0;
+	const unsigned char *inptr;
+	int digit, n = 0;
 	
-	decode_lwsp (&inptr);
+	decode_lwsp (in);
 	
+	inptr = (const unsigned char *) *in;
 	while (isdigit ((int) *inptr)) {
-		n = n * 10 + (*inptr - '0');
+		digit = (*inptr - '0');
+		if (INT_OVERFLOW (n, digit)) {
+			while (isdigit ((int) *inptr))
+				inptr++;
+			break;
+		}
+		
+		n = (n * 10) + digit;
+		
 		inptr++;
 	}
 	
-	*in = inptr;
+	*in = (const char *) inptr;
 	
 	return n;
 }
@@ -125,29 +120,47 @@ static char *
 decode_quoted_string (const char **in)
 {
 	const char *start, *inptr = *in;
-	char *out = NULL;
+	char *outptr, *out = NULL;
+	gboolean unescape = FALSE;
 	
 	decode_lwsp (&inptr);
-	if (*inptr == '"') {
-		start = inptr++;
-		
-		while (*inptr && *inptr != '"') {
-			if (*inptr++ == '\\')
-				inptr++;
-		}
-		
-		if (*inptr == '"') {
-			start++;
-			out = g_strndup (start, (unsigned int) (inptr - start));
+	
+	if (*inptr != '"') {
+		*in = inptr;
+		return NULL;
+	}
+	
+	start = inptr++;
+	
+	while (*inptr && *inptr != '"') {
+		if (*inptr++ == '\\') {
+			unescape = TRUE;
 			inptr++;
-		} else {
-			/* string wasn't properly quoted */
-			out = g_strndup (start, (unsigned int) (inptr - start));
 		}
 	}
 	
+	if (*inptr == '"') {
+		start++;
+		out = g_strndup (start, (unsigned int) (inptr - start));
+		inptr++;
+	} else {
+		/* string wasn't properly quoted */
+		out = g_strndup (start, (unsigned int) (inptr - start));
+	}
+	
 	*in = inptr;
 	
+	if (unescape) {
+		inptr = outptr = out;
+		while (*inptr) {
+			if (*inptr == '\\')
+				inptr++;
+			*outptr++ = *inptr++;
+		}
+		
+		*outptr = '\0';
+	}
+	
 	return out;
 }
 
@@ -268,10 +281,10 @@ decode_param (const char **in, char **paramp, char **valuep, int *id, gboolean *
 		value = decode_value (&inptr);
 		
 		if (!is_rfc2184 && value) {
-			if (!strncmp (value, "=?", 2)) {
-				/* We have a broken param value that is rfc2047 encoded.
-				 * Since both Outlook and Netscape/Mozilla do this, we
-				 * should handle this case.
+			if (strstr (value, "=?") != NULL) {
+				/* We (may) have a broken param value that is rfc2047
+				 * encoded. Since both Outlook and Netscape/Mozilla do
+				 * this, we should handle this case.
 				 */
 				
 				if ((val = g_mime_utils_header_decode_text (value))) {
@@ -334,16 +347,11 @@ rfc2184_sort_cb (const void *v0, const void *v1)
 #define HEXVAL(c) (isdigit (c) ? (c) - '0' : tolower (c) - 'a' + 10)
 
 static size_t
-hex_decode (const unsigned char *in, size_t len, unsigned char *out)
+hex_decode (const char *in, size_t len, char *out)
 {
-	register const unsigned char *inptr;
-	register unsigned char *outptr;
-	const unsigned char *inend;
-	
-	inptr = in;
-	inend = in + len;
-	
-	outptr = out;
+	register const unsigned char *inptr = (const unsigned char *) in;
+	register unsigned char *outptr = (unsigned char *) out;
+	const unsigned char *inend = inptr + len;
 	
 	while (inptr < inend) {
 		if (*inptr == '%') {
@@ -358,7 +366,7 @@ hex_decode (const unsigned char *in, size_t len, unsigned char *out)
 	
 	*outptr = '\0';
 	
-	return outptr - out;
+	return ((char *) outptr) - out;
 }
 
 static const char *
@@ -605,7 +613,7 @@ decode_param_list (const char *in)
  *
  * Creates a parameter list based on the input string.
  *
- * Returns a GMimeParam structure based on @string.
+ * Returns a #GMimeParam structure based on @string.
  **/
 GMimeParam *
 g_mime_param_new_from_string (const char *string)
@@ -701,21 +709,24 @@ g_mime_param_append_param (GMimeParam *params, GMimeParam *param)
 
 /* FIXME: I wrote this in a quick & dirty fasion - it may not be 100% correct */
 static char *
-encode_param (const unsigned char *in, gboolean *encoded)
+encode_param (const char *in, gboolean *encoded)
 {
-	register const unsigned char *inptr;
-	unsigned char *outbuf = NULL;
+	register const unsigned char *inptr = (const unsigned char *) in;
+	const unsigned char *instart = inptr;
 	iconv_t cd = (iconv_t) -1;
 	const char *charset = NULL;
+	char *outbuf = NULL;
 	unsigned char c;
 	char *outstr;
 	GString *out;
 	
 	*encoded = FALSE;
 	
-	for (inptr = in; *inptr && inptr - in < GMIME_FOLD_LEN; inptr++)
+	while (*inptr && ((inptr - instart) < GMIME_FOLD_LEN)) {
 		if (*inptr > 127)
 			break;
+		inptr++;
+	}
 	
 	if (*inptr == '\0')
 		return g_strdup (in);
@@ -734,13 +745,13 @@ encode_param (const unsigned char *in, gboolean *encoded)
 		g_mime_iconv_close (cd);
 		if (outbuf == NULL) {
 			charset = "UTF-8";
-			inptr = in;
+			inptr = instart;
 		} else {
-			inptr = outbuf;
+			inptr = (const unsigned char *) outbuf;
 		}
 	} else {
 		charset = "UTF-8";
-		inptr = in;
+		inptr = instart;
 	}
 	
 	/* FIXME: set the 'language' as well, assuming we can get that info...? */
