@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -34,6 +35,20 @@
 
 #include "gmime-stream-mmap.h"
 
+
+/**
+ * SECTION: gmime-stream-mmap
+ * @title: GMimeStreamMmap
+ * @short_description: A memory-mapped file stream
+ * @see_also: #GMimeStream
+ *
+ * A #GMimeStream implementation using a memory-mapped file backing
+ * store. This may be faster than #GMimeStreamFs or #GMimeStreamFile
+ * but you'll have to do your own performance checking to be sure for
+ * your particular application/platform.
+ **/
+
+
 static void g_mime_stream_mmap_class_init (GMimeStreamMmapClass *klass);
 static void g_mime_stream_mmap_init (GMimeStreamMmap *stream, GMimeStreamMmapClass *klass);
 static void g_mime_stream_mmap_finalize (GObject *object);
@@ -67,7 +82,7 @@ g_mime_stream_mmap_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeStreamMmap),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_stream_mmap_init,
 		};
 		
@@ -210,8 +225,7 @@ stream_close (GMimeStream *stream)
 	}
 	
 	if (mstream->owner && mstream->fd != -1) {
-		ret = close (mstream->fd);
-		if (ret != -1)
+		if ((ret = close (mstream->fd)) != -1)
 			mstream->fd = -1;
 	}
 	
@@ -233,9 +247,9 @@ stream_reset (GMimeStream *stream)
 {
 	GMimeStreamMmap *mstream = (GMimeStreamMmap *) stream;
 	
-	g_return_val_if_fail (mstream->fd != -1, -1);
+	if (mstream->fd == -1)
+		return -1;
 	
-	stream->position = stream->bound_start;
 	mstream->eos = FALSE;
 	
 	return 0;
@@ -258,7 +272,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 		break;
 	case GMIME_STREAM_SEEK_END:
 		if (stream->bound_end == -1) {
-			real = offset <= 0 ? stream->bound_start + mstream->maplen + offset : -1;
+			real = offset <= 0 ? stream->bound_start + (off_t) mstream->maplen + offset : -1;
 			if (real != -1) {
 				if (real < stream->bound_start)
 					real = stream->bound_start;
@@ -271,11 +285,16 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 		break;
 	}
 	
-	if (stream->bound_end != -1)
-		real = MIN (real, stream->bound_end);
-	real = MAX (real, stream->bound_start);
+	/* sanity check the resultant offset */
+	if (real < stream->bound_start)
+		return -1;
 	
-	if (real != stream->position && mstream->eos)
+	if (stream->bound_end != -1 && real > stream->bound_end)
+		return -1;
+	
+	/* reset eos if appropriate */
+	if ((stream->bound_end != -1 && real < stream->bound_end) ||
+	    (mstream->eos && real < stream->position))
 		mstream->eos = FALSE;
 	
 	stream->position = real;
@@ -297,7 +316,7 @@ stream_length (GMimeStream *stream)
 	if (stream->bound_start != -1 && stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
-	return mstream->maplen;
+	return mstream->maplen - stream->bound_start;
 }
 
 static GMimeStream *
@@ -306,7 +325,7 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
 	/* FIXME: maybe we should return a GMimeStreamFs? */
 	GMimeStreamMmap *mstream;
 	
-	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL, NULL);
+	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL);
 	mstream->owner = FALSE;
 	mstream->fd = GMIME_STREAM_MMAP (stream)->fd;
 	
@@ -349,7 +368,7 @@ g_mime_stream_mmap_new (int fd, int prot, int flags)
 	if (map == MAP_FAILED)
 		return NULL;
 	
-	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL, NULL);
+	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL);
 	mstream->owner = TRUE;
 	mstream->eos = FALSE;
 	mstream->fd = fd;
@@ -396,7 +415,7 @@ g_mime_stream_mmap_new_with_bounds (int fd, int prot, int flags, off_t start, of
 	if (map == MAP_FAILED)
 		return NULL;
 	
-	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL, NULL);
+	mstream = g_object_new (GMIME_TYPE_STREAM_MMAP, NULL);
 	mstream->owner = TRUE;
 	mstream->eos = FALSE;
 	mstream->fd = fd;
