@@ -1,23 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
+/*  GMime
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
- *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -31,8 +29,9 @@
 
 #include "internet-address.h"
 #include "gmime-table-private.h"
-#include "gmime-utils.h"
+#include "gmime-parse-utils.h"
 #include "gmime-iconv-utils.h"
+#include "gmime-utils.h"
 
 
 #ifdef ENABLE_WARNINGS
@@ -43,10 +42,16 @@
 
 #define d(x)
 
-/* public functions that we don't care for the user of GMime to know or care about */
-void decode_lwsp (const char **in);
-char *decode_word (const char **in);
-char *decode_addrspec (const char **in);
+
+/**
+ * SECTION: internet-address
+ * @title: InternetAddress
+ * @short_description: Internet addresses
+ * @see_also:
+ *
+ * An #InternetAddress represents what is commonly referred to as an
+ * E-Mail address.
+ **/
 
 
 /**
@@ -57,7 +62,7 @@ char *decode_addrspec (const char **in);
  * Returns a new #InternetAddress object.
  **/
 InternetAddress *
-internet_address_new ()
+internet_address_new (void)
 {
 	InternetAddress *ia;
 	
@@ -80,17 +85,15 @@ internet_address_new ()
 static void
 internet_address_destroy (InternetAddress *ia)
 {
-	if (ia) {
-		g_free (ia->name);
-		
-		if (ia->type == INTERNET_ADDRESS_GROUP) {
-			internet_address_list_destroy (ia->value.members);
-		} else {
-			g_free (ia->value.addr);
-		}
-		
-		g_free (ia);
+	g_free (ia->name);
+	
+	if (ia->type == INTERNET_ADDRESS_GROUP) {
+		internet_address_list_destroy (ia->value.members);
+	} else {
+		g_free (ia->value.addr);
 	}
+	
+	g_free (ia);
 }
 
 
@@ -227,8 +230,6 @@ internet_address_set_addr (InternetAddress *ia, const char *addr)
 void
 internet_address_set_group (InternetAddress *ia, InternetAddressList *group)
 {
-	/*InternetAddressList *members, *next; */
-	
 	g_return_if_fail (ia != NULL);
 	g_return_if_fail (ia->type != INTERNET_ADDRESS_NAME);
 	
@@ -261,13 +262,16 @@ internet_address_add_member (InternetAddress *ia, InternetAddress *member)
  * internet_address_get_type:
  * @ia: internet address
  *
- * Returns the type of the internet address.
+ * Gets the type of the internet address, which will either be
+ * #INTERNET_ADDRESS_NAME or #INTERNET_ADDRESS_GROUP.
+ *
+ * Returns the type of @ia.
  **/
 InternetAddressType
 internet_address_get_type (InternetAddress *ia)
 {
 	g_return_val_if_fail (ia != NULL, INTERNET_ADDRESS_NONE);
-
+	
 	return ia->type;
 }
 
@@ -275,13 +279,16 @@ internet_address_get_type (InternetAddress *ia)
  * internet_address_get_name:
  * @ia: internet address
  *
- * Returns the name of the internet address.
+ * Gets the name component of the internet address. If the internet
+ * address is a group, it will get the group name.
+ *
+ * Returns the name of @ia.
  **/
 const char *
 internet_address_get_name (InternetAddress *ia)
 {
 	g_return_val_if_fail (ia != NULL, NULL);
-
+	
 	return ia->name;
 }
 
@@ -290,14 +297,16 @@ internet_address_get_name (InternetAddress *ia)
  * internet_address_get_addr:
  * @ia: internet address
  *
- * Returns the address of the internet address.
+ * Gets the addr-spec of the internet address.
+ *
+ * Returns the address of @ia.
  **/
 const char *
 internet_address_get_addr (InternetAddress *ia)
 {
 	g_return_val_if_fail (ia != NULL, NULL);
 	g_return_val_if_fail (ia->type != INTERNET_ADDRESS_GROUP, NULL);
-
+	
 	return ia->value.addr;
 }
 
@@ -306,14 +315,17 @@ internet_address_get_addr (InternetAddress *ia)
  * internet_address_get_members:
  * @ia: internet address
  *
- * Returns the list of internet addresses
+ * Gets the #InternetAddressList containing the group members of an
+ * rfc822 group address.
+ *
+ * Returns the members of @ia.
  **/
 const InternetAddressList *
 internet_address_get_members (InternetAddress *ia)
 {
 	g_return_val_if_fail (ia != NULL, NULL);
 	g_return_val_if_fail (ia->type != INTERNET_ADDRESS_NAME, NULL);
-
+	
 	return ia->value.members;
 }
 
@@ -323,7 +335,7 @@ internet_address_get_members (InternetAddress *ia)
  * @list: a list of internet addresses
  * @ia: internet address to prepend
  *
- * Prepends the internet address to the list of internet addresses
+ * Prepends the internet address @ia to the list of internet addresses
  * pointed to by @list.
  *
  * Returns the resultant list.
@@ -337,8 +349,8 @@ internet_address_list_prepend (InternetAddressList *list, InternetAddress *ia)
 	
 	internet_address_ref (ia);
 	node = g_new (InternetAddressList, 1);
-	node->next = list;
 	node->address = ia;
+	node->next = list;
 	
 	return node;
 }
@@ -426,9 +438,9 @@ internet_address_list_concat (InternetAddressList *a, InternetAddressList *b)
  * internet_address_list_next:
  * @list: list of internet addresses
  *
- * Advances to the next address node in the InternetAddessList.
+ * Advances to the next address node in the #InternetAddessList.
  *
- * Returns the next address node in the InternetAddessList.
+ * Returns the next address node in the #InternetAddessList.
  **/
 InternetAddressList *
 internet_address_list_next (const InternetAddressList *list)
@@ -441,15 +453,14 @@ internet_address_list_next (const InternetAddressList *list)
  * internet_address_list_get_address:
  * @list: list of internet addresses
  *
+ * Gets the #InternetAddress currently pointed to in @list.
+ *
  * Returns the #InternetAddress currently pointed to in @list.
  **/
 InternetAddress *
 internet_address_list_get_address (const InternetAddressList *list)
 {
-	if (!list)
-		return NULL;
-
-	return list->address;
+	return list ? list->address : NULL;
 }
 
 /**
@@ -513,18 +524,167 @@ encoded_name (const char *raw, gboolean rfc2047_encode)
 	return name;
 }
 
+enum {
+	INTERNET_ADDRESS_ENCODE = 1 << 0,
+	INTERNET_ADDRESS_FOLD   = 1 << 1,
+};
+
+static void _internet_address_list_to_string (const InternetAddressList *list, guint32 flags, size_t *linelen, GString *string);
+
 static void
-internet_address_list_to_string_internal (const InternetAddressList *list, gboolean encode, GString *string)
+linewrap (GString *string)
+{
+	if (string->len > 0 && string->str[string->len - 1] == ' ') {
+		string->str[string->len - 1] = '\n';
+		g_string_append_c (string, '\t');
+	} else {
+		g_string_append (string, "\n\t");
+	}
+}
+
+static void
+append_folded_name (GString *string, size_t *linelen, const char *name)
+{
+	const char *word, *lwsp;
+	size_t len;
+	
+	word = name;
+	
+	while (*word) {
+		lwsp = word;
+		
+		if (*word == '"') {
+			/* quoted string, don't break these up */
+			lwsp++;
+			
+			while (*lwsp && *lwsp != '"') {
+				if (*lwsp == '\\')
+					lwsp++;
+				
+				if (*lwsp)
+					lwsp++;
+			}
+			
+			if (*lwsp == '"')
+				lwsp++;
+		} else {
+			/* normal word */
+			while (*lwsp && !is_lwsp (*lwsp))
+				lwsp++;
+		}
+		
+		len = lwsp - word;
+		if (*linelen > 1 && (*linelen + len) > GMIME_FOLD_LEN) {
+			linewrap (string);
+			*linelen = 1;
+		}
+		
+		g_string_append_len (string, word, len);
+		*linelen += len;
+		
+		word = lwsp;
+		while (*word && is_lwsp (*word))
+			word++;
+		
+		if (*word && is_lwsp (*lwsp)) {
+			g_string_append_c (string, ' ');
+			(*linelen)++;
+		}
+	}
+}
+
+static void
+_internet_address_to_string (const InternetAddress *ia, guint32 flags, size_t *linelen, GString *string)
+{
+	gboolean encode = flags & INTERNET_ADDRESS_ENCODE;
+	gboolean fold = flags & INTERNET_ADDRESS_FOLD;
+	char *name;
+	size_t len;
+	
+	if (ia->type == INTERNET_ADDRESS_NAME) {
+		if (ia->name && *ia->name) {
+			name = encoded_name (ia->name, encode);
+			len = strlen (name);
+			
+			if (fold && (*linelen + len) > GMIME_FOLD_LEN) {
+				if (len > GMIME_FOLD_LEN) {
+					/* we need to break up the name */
+					append_folded_name (string, linelen, name);
+				} else {
+					/* the name itself is short enough to fit on a single
+					 * line, but only if we write it on a line by itself */
+					if (*linelen > 1) {
+						linewrap (string);
+						*linelen = 1;
+					}
+					
+					g_string_append_len (string, name, len);
+					*linelen += len;
+				}
+			} else {
+				/* we can safly fit the name on this line */
+				g_string_append_len (string, name, len);
+				*linelen += len;
+			}
+			
+			g_free (name);
+			
+			len = strlen (ia->value.addr);
+			
+			if (fold && (*linelen + len + 3) >= GMIME_FOLD_LEN) {
+				g_string_append_len (string, "\n\t<", 3);
+				*linelen = 2;
+			} else {
+				g_string_append_len (string, " <", 2);
+				*linelen += 2;
+			}
+			
+			g_string_append_len (string, ia->value.addr, len);
+			g_string_append_c (string, '>');
+			*linelen += len + 1;
+		} else {
+			len = strlen (ia->value.addr);
+			
+			if (fold && (*linelen + len) > GMIME_FOLD_LEN) {
+				linewrap (string);
+				*linelen = 1;
+			}
+			
+			g_string_append_len (string, ia->value.addr, len);
+			*linelen += len;
+		}
+	} else if (ia->type == INTERNET_ADDRESS_GROUP) {
+		InternetAddressList *members;
+		
+		name = encoded_name (ia->name, encode);
+		len = strlen (name);
+		
+		if (fold && *linelen > 1 && (*linelen + len + 1) > GMIME_FOLD_LEN) {
+			linewrap (string);
+			*linelen = 1;
+		}
+		
+		g_string_append_len (string, name, len);
+		g_string_append_len (string, ": ", 2);
+		*linelen += len + 2;
+		g_free (name);
+		
+		members = ia->value.members;
+		_internet_address_list_to_string (members, flags, linelen, string);
+		g_string_append_c (string, ';');
+		*linelen += 1;
+	}
+}
+
+static void
+_internet_address_list_to_string (const InternetAddressList *list, guint32 flags, size_t *linelen, GString *string)
 {
 	while (list) {
-		char *addr;
+		_internet_address_to_string (list->address, flags, linelen, string);
 		
-		addr = internet_address_to_string (list->address, encode);
-		if (addr) {
-			g_string_append (string, addr);
-			g_free (addr);
-			if (list->next)
-				g_string_append (string, ", ");
+		if (list->next) {
+			g_string_append (string, ", ");
+			*linelen += 2;
 		}
 		
 		list = list->next;
@@ -546,32 +706,16 @@ internet_address_list_to_string_internal (const InternetAddressList *list, gbool
 char *
 internet_address_to_string (const InternetAddress *ia, gboolean encode)
 {
-	char *name, *str = NULL;
+	guint32 flags = encode ? INTERNET_ADDRESS_ENCODE : 0;
+	size_t linelen = 0;
+	GString *string;
+	char *str;
 	
-	if (ia->type == INTERNET_ADDRESS_NAME) {
-		if (ia->name && *ia->name) {
-			name = encoded_name (ia->name, encode);
-			str = g_strdup_printf ("%s <%s>", name, ia->value.addr);
-			g_free (name);
-		} else {
-			str = g_strdup (ia->value.addr);
-		}
-	} else if (ia->type == INTERNET_ADDRESS_GROUP) {
-		InternetAddressList *members;
-		GString *string;
-		
-		name = encoded_name (ia->name, encode);
-		string = g_string_new (name);
-		g_string_append (string, ": ");
-		g_free (name);
-		
-		members = ia->value.members;
-		internet_address_list_to_string_internal (members, encode, string);
-		g_string_append (string, ";");
-		
-		str = string->str;
-		g_string_free (string, FALSE);
-	}
+	string = g_string_new ("");
+	_internet_address_to_string (ia, flags, &linelen, string);
+	str = string->str;
+	
+	g_string_free (string, FALSE);
 	
 	return str;
 }
@@ -590,273 +734,36 @@ internet_address_to_string (const InternetAddress *ia, gboolean encode)
 char *
 internet_address_list_to_string (const InternetAddressList *list, gboolean encode)
 {
+	guint32 flags = encode ? INTERNET_ADDRESS_ENCODE : 0;
+	size_t linelen = 0;
 	GString *string;
 	char *str;
 	
 	string = g_string_new ("");
-	internet_address_list_to_string_internal (list, encode, string);
+	_internet_address_list_to_string (list, flags, &linelen, string);
 	str = string->str;
+	
 	g_string_free (string, FALSE);
 	
 	return str;
 }
 
 
+/**
+ * internet_address_list_writer:
+ * @list: list of internet addresses
+ * @str: string to write to
+ *
+ * Writes the rfc2047-encoded rfc822 formatted addresses in @list to
+ * @string, folding appropriately.
+ **/
 void
-decode_lwsp (const char **in)
+internet_address_list_writer (const InternetAddressList *list, GString *str)
 {
-	const char *inptr = *in;
+	guint32 flags = INTERNET_ADDRESS_ENCODE | INTERNET_ADDRESS_FOLD;
+	size_t linelen = str->len;
 	
-	while (*inptr && (*inptr == '(' || is_lwsp (*inptr))) {
-		while (*inptr && is_lwsp (*inptr))
-			inptr++;
-		
-		/* skip over any comments */
-		if (*inptr == '(') {
-			int depth = 1;
-			
-			inptr++;
-			while (*inptr && depth) {
-				if (*inptr == '\\' && *(inptr + 1))
-					inptr++;
-				else if (*inptr == '(')
-					depth++;
-				else if (*inptr == ')')
-					depth--;
-				
-				inptr++;
-			}
-		}
-	}
-	
-	*in = inptr;
-}
-
-static char *
-decode_quoted_string (const char **in)
-{
-	const char *inptr = *in;
-	char *out = NULL;
-	
-	decode_lwsp (&inptr);
-	if (*inptr == '"') {
-		out = (char *) inptr;
-		
-		inptr++;
-		while (*inptr && *inptr != '"') {
-			if (*inptr == '\\')
-				inptr++;
-			
-			if (*inptr)
-				inptr++;
-		}
-		
-		if (*inptr == '"')
-			inptr++;
-		
-		out = g_strndup (out, inptr - out);
-	}
-	
-	*in = inptr;
-	
-	return out;
-}
-
-static char *
-decode_atom (const char **in)
-{
-	const char *inptr = *in, *start;
-	
-	decode_lwsp (&inptr);
-	start = inptr;
-	while (is_atom (*inptr))
-		inptr++;
-	*in = inptr;
-	if (inptr > start)
-		return g_strndup (start, inptr - start);
-	else
-		return NULL;
-}
-
-char *
-decode_word (const char **in)
-{
-	const char *inptr = *in;
-	
-	decode_lwsp (&inptr);
-	if (*inptr == '"') {
-		*in = inptr;
-		return decode_quoted_string (in);
-	} else {
-		*in = inptr;
-		return decode_atom (in);
-	}
-}
-
-static gboolean
-decode_subliteral (const char **in, GString *domain)
-{
-	const char *inptr = *in;
-	gboolean got = FALSE;
-	
-	while (*inptr && *inptr != '.' && *inptr != ']') {
-		if (is_dtext (*inptr)) {
-			g_string_append_c (domain, *inptr);
-			inptr++;
-			got = TRUE;
-		} else if (is_lwsp (*inptr))
-			decode_lwsp (&inptr);
-		else
-			break;
-	}
-	
-	*in = inptr;
-	
-	return got;
-}
-
-static void
-decode_domain_literal (const char **in, GString *domain)
-{
-	const char *inptr = *in;
-	
-	decode_lwsp (&inptr);
-	while (*inptr && *inptr != ']') {
-		if (decode_subliteral (&inptr, domain) && *inptr == '.') {
-			g_string_append_c (domain, *inptr);
-			inptr++;
-		} else if (*inptr != ']') {
-			w(g_warning ("Malformed domain-literal, unexpected char '%c': %s",
-				     *inptr, *in));
-			
-			/* try and skip to the next char ?? */
-			inptr++;
-		}
-	}
-	
-	*in = inptr;
-}
-
-static char *
-decode_domain (const char **in)
-{
-	const char *inptr, *save;
-	GString *domain;
-	char *dom, *atom;
-	
-	domain = g_string_new ("");
-	
-	inptr = *in;
-	while (inptr && *inptr) {
-		decode_lwsp (&inptr);
-		if (*inptr == '[') {
-			/* domain literal */
-			g_string_append_c (domain, '[');
-			inptr++;
-			
-			decode_domain_literal (&inptr, domain);
-			
-			if (*inptr == ']') {
-				g_string_append_c (domain, ']');
-				inptr++;
-			} else
-				w(g_warning ("Missing ']' in domain-literal: %s", *in));
-		} else {
-			if (!(atom = decode_atom (&inptr))) {
-				w(g_warning ("Unexpected char '%c' in domain: %s", *inptr, *in));
-				/* remove the last '.' */
-				if (domain->len && domain->str[domain->len - 1] == '.')
-					g_string_truncate (domain, domain->len - 1);
-				break;
-			}
-			
-			g_string_append (domain, atom);
-			g_free (atom);
-		}
-		
-		save = inptr;
-		decode_lwsp (&inptr);
-		if (*inptr != '.') {
-			inptr = save;
-			break;
-		}
-		
-		g_string_append_c (domain, '.');
-		inptr++;
-	}
-	
-	if (domain->len)
-		dom = domain->str;
-	else
-		dom = NULL;
-	
-	g_string_free (domain, dom ? FALSE : TRUE);
-	
-	*in = inptr;
-	
-	return dom;
-}
-
-char *
-decode_addrspec (const char **in)
-{
-	char *domain, *word, *str = NULL;
-	const char *inptr;
-	GString *addrspec;
-	
-	decode_lwsp (in);
-	inptr = *in;
-	
-	if (!(word = decode_word (&inptr))) {
-		w(g_warning ("No local-part in addr-spec: %s", *in));
-		return NULL;
-	}
-	
-	addrspec = g_string_new (word);
-	g_free (word);
-	
-	/* get the rest of the local-part */
-	decode_lwsp (&inptr);
-	while (*inptr == '.') {
-		g_string_append_c (addrspec, *inptr++);
-		word = decode_word (&inptr);
-		if (word) {
-			g_string_append (addrspec, word);
-			decode_lwsp (&inptr);
-			g_free (word);
-		} else {
-			w(g_warning ("Invalid local-part in addr-spec: %s", *in));
-			goto exception;
-		}
-	}
-	
-	/* we should be at the '@' now... */
-	if (*inptr++ != '@') {
-		w(g_warning ("Invalid addr-spec; missing '@': %s", *in));
-		goto exception;
-	}
-	
-	if (!(domain = decode_domain (&inptr))) {
-		w(g_warning ("No domain in addr-spec: %s", *in));
-		goto exception;
-	}
-	
-	g_string_append_c (addrspec, '@');
-	g_string_append (addrspec, domain);
-	g_free (domain);
-	
-	str = addrspec->str;
-	g_string_free (addrspec, FALSE);
-	
-	*in = inptr;
-	
-	return str;
-	
- exception:
-	
-	g_string_free (addrspec, TRUE);
-	
-	return NULL;
+	_internet_address_list_to_string (list, flags, &linelen, str);
 }
 
 static InternetAddress *
@@ -886,8 +793,7 @@ decode_mailbox (const char **in)
 			g_string_append (name, pre);
 			g_free (pre);
 		retry:
-			pre = decode_word (&inptr);
-			if (pre)
+			if ((pre = decode_word (&inptr)))
 				g_string_append_c (name, ' ');
 		}
 		
@@ -897,7 +803,7 @@ decode_mailbox (const char **in)
 			bracket = TRUE;
 			pre = decode_word (&inptr);
 		} else if (!retried && *inptr) {
-			w(g_warning ("Unexpected char '%c' in address: %s: attempting recovery.",
+			w(g_warning ("Unexpected char '%c' in mailbox: %s: attempting recovery.",
 				     *inptr, *in));
 			/* chew up this bad char and then attempt 1 more pass at parsing */
 			g_string_append_c (name, *inptr++);
@@ -919,7 +825,12 @@ decode_mailbox (const char **in)
 		if (name)
 			g_string_free (name, TRUE);
 		g_string_free (addr, TRUE);
-		*in = inptr + 1;
+		
+		/* comma will be eaten by our caller */
+		if (*inptr && *inptr != ',')
+			*in = inptr + 1;
+		else
+			*in = inptr;
 		
 		return NULL;
 	}
@@ -929,8 +840,7 @@ decode_mailbox (const char **in)
 	while (*inptr == '.' && pre) {
 		inptr++;
 		g_free (pre);
-		pre = decode_word (&inptr);
-		if (pre) {
+		if ((pre = decode_word (&inptr))) {
 			g_string_append_c (addr, '.');
 			g_string_append (addr, pre);
 		}
@@ -943,8 +853,7 @@ decode_mailbox (const char **in)
 		char *domain;
 		
 		inptr++;
-		domain = decode_domain (&inptr);
-		if (domain) {
+		if ((domain = decode_domain (&inptr))) {
 			g_string_append_c (addr, '@');
 			g_string_append (addr, domain);
 			g_free (domain);
@@ -973,8 +882,7 @@ decode_mailbox (const char **in)
 		comment = (char *) inptr;
 		decode_lwsp (&inptr);
 		if (inptr > comment) {
-			comment = memchr (comment, '(', inptr - comment);
-			if (comment) {
+			if ((comment = memchr (comment, '(', inptr - comment))) {
 				const char *cend;
 				
 				/* find the end of the comment */
@@ -998,22 +906,12 @@ decode_mailbox (const char **in)
 	
 	if (addr->len) {
 		if (name && !g_utf8_validate (name->str, -1, NULL)) {
-			/* A (broken) mailer has sent us an unencoded
-			 * 8bit value (and it doesn't seem to be valid
-			 * UTF-8 either).  Attempt to save it by
-			 * assuming it's in the user's locale and
-			 * converting to UTF-8 */
-			char *buf;
+			/* A (broken) mailer has sent us raw 8bit/multibyte text data... */
+			char *utf8 = g_mime_utils_decode_8bit (name->str, name->len);
 			
-			buf = g_mime_iconv_locale_to_utf8 (name->str);
-			if (buf) {
-				g_string_truncate (name, 0);
-				g_string_append (name, buf);
-				g_free (buf);
-			} else {
-				d(g_warning ("Failed to convert \"%s\" to UTF-8: %s",
-					     name->str, g_strerror (errno)));
-			}
+			g_string_truncate (name, 0);
+			g_string_append (name, utf8);
+			g_free (utf8);
 		}
 		
 		mailbox = internet_address_new_name (name ? name->str : NULL, addr->str);
@@ -1044,8 +942,7 @@ decode_address (const char **in)
 		g_string_append (name, pre);
 		g_free (pre);
 		
-		pre = decode_word (&inptr);
-		if (pre)
+		if ((pre = decode_word (&inptr)))
 			g_string_append_c (name, ' ');
 	}
 	
@@ -1063,8 +960,7 @@ decode_address (const char **in)
 		while (*inptr && *inptr != ';') {
 			InternetAddress *member;
 			
-			member = decode_mailbox (&inptr);
-			if (member) {
+			if ((member = decode_mailbox (&inptr))) {
 				tail->next = g_new (InternetAddressList, 1);
 				tail = tail->next;
 				tail->next = NULL;
@@ -1075,8 +971,7 @@ decode_address (const char **in)
 			while (*inptr == ',') {
 				inptr++;
 				decode_lwsp (&inptr);
-				member = decode_mailbox (&inptr);
-				if (member) {
+				if ((member = decode_mailbox (&inptr))) {
 					tail->next = g_new (InternetAddressList, 1);
 					tail = tail->next;
 					tail->next = NULL;
@@ -1151,8 +1046,7 @@ internet_address_parse_string (const char *string)
 		else if (*inptr) {
 			w(g_warning ("Parse error at '%s': expected ','", inptr));
 			/* try skipping to the next address */
-			inptr = strchr (inptr, ',');
-			if (inptr)
+			if ((inptr = strchr (inptr, ',')))
 				inptr++;
 		}
 	}
